<?php

/**
 * @copyright (c) 2020.
 * @author            Alan Fuller (support@fullworks)
 * @licence           GPL V3 https://www.gnu.org/licenses/gpl-3.0.en.html
 * @link                  https://fullworks.net
 *
 * This file is part of  a Fullworks plugin.
 *
 *   This plugin is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This plugin is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with  this plugin.  https://www.gnu.org/licenses/gpl-3.0.en.html
 */
namespace Quick_Event_Manager\UI\Admin;

class Admin
{
    private  $plugin_name ;
    private  $version ;
    /**
     * @param \Freemius $freemius Object for freemius.
     */
    private  $freemius ;
    public function __construct( $plugin_name, $version, $freemius )
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->freemius = $freemius;
    }
    
    public function hooks()
    {
        // @TODO check if style / js actually needed.
        //add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_styles' ) );
        //add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
        add_action( 'admin_notices', array( $this, 'admin_notice_freemius' ) );
        add_action( 'init', array( $this, 'generate_freemius_licence' ) );
    }
    
    public function enqueue_styles()
    {
        /**
         * @TODO decide if admin styles needed
         */
        wp_enqueue_style(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . 'css/admin.css',
            array(),
            $this->version,
            'all'
        );
    }
    
    public function enqueue_scripts()
    {
        /**
         * @TODO decide if admin scripts required.
         */
        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . 'js/admin.js',
            array( 'jquery' ),
            $this->version,
            false
        );
    }
    
    public function admin_notice_freemius()
    {
        // Don't display notices to users that can't do anything about it.
        if ( !current_user_can( 'install_plugins' ) ) {
            return;
        }
        // Notices are only displayed on the dashboard, plugins, tools, and settings admin pages.
        $page = get_current_screen()->base;
        $display_on_pages = array(
            'dashboard',
            'plugins',
            'tools',
            'options-general'
        );
        $display = false;
        if ( preg_match( '#quick-event-manager#i', $page ) ) {
            $display = true;
        }
        if ( in_array( $page, $display_on_pages, true ) ) {
            $display = true;
        }
        if ( !$display ) {
            return;
        }
        $qem_freemius_licence = get_option( 'qem_licence_generated', false );
        $qpp_key = get_option( 'qpp_key', array(
            'authorised' => false,
        ) );
        $user = wp_get_current_user();
        $freemius_nonce = wp_create_nonce( 'qem_freemius_licence' );
        $notice = '';
        $logo = '<img style="float:left;padding-right: 10px" height="64px" src="https://ps.w.org/quick-event-manager/assets/icon-128x128.png">';
        global  $qem_fs ;
        if ( $qpp_key['authorised'] && $qem_fs->is_free_plan() ) {
            
            if ( 'generated' !== $qem_freemius_licence ) {
                $failed = '';
                if ( 'failed' === $qem_freemius_licence ) {
                    $failed = '<p><strong style="color: red;">Licence Generation Failed</strong>, make sure you are using the latest version and try again, 
if after several attempts you still get this message contact me at 
<a target="_blank" href="mailto:support@fullworks.net">support</a> with your details, name , email, domain name etc</p>';
                }
                $notice .= sprintf(
                    __( '%1$s<strong>Important action required</strong>. %2$s<p>Big changes are coming to Quick Event Manager and you MUST take action now to keep your premium features!!<br>
 You will need to <strong>download and install the PREMIUM</strong> plugin to keep your premium features. Don\'t worry it is easy.</p>
<p></p><a target="_blank" href="%3$s" class="button" >CLICK HERE TO GET YOUR DOWNLOAD AND LICENCE BY EMAIL</a></p><p>If you do not download and install the premium plugin 
- when a future version is released you may lose features you depend on, so register now and be prepared.</p>
<p>The email will be sent to <strong>%4$s</strong> If you don\'t get the email soon, check your spam </p><p>To be clear if you already have paid for the Pro version - you do not have to pay again</p>', 'quick-event-manager' ),
                    $logo,
                    $failed,
                    '?action=freemius&_wpnonce=' . $freemius_nonce,
                    $user->data->user_email
                );
            } else {
                $notice .= sprintf( __( '%1$s<strong>Important action required - Your licence request was successful</strong>. <p>Big changes are coming to Quick PayPal Payments  and you MUST take action now to keep your premium features!!<br>
 You have requested the PREMIUM plugin, please install it to keep your premium features. Don\'t worry it is easy.
</p><p>If you do not download and install the premium plugin - when a future version is released you may lose features you depend on, so register now and be prepared.
The email was sent to <strong>%2$s</strong>. </p><p>If you do not have access to <strong>%2$s</strong>( check your spam ) contact me at 
<a target="_blank" href="mailto:support@fullworks.net">support</a> with your details.</p><p>You can also use your email <strong>%2$s</strong>  to login at <a href="https://fullworks.net/account/" target="_blank">
https://fullworks.net/account/</a> to get your download and licence key
</p>', 'quick-event-manager' ), $logo, $user->data->user_email );
            }
        
        }
        // Output notice HTML.
        if ( !empty($notice) ) {
            printf( '<div id="message" class="notice notice-warning" style="overflow:hidden;font-size: 150%;"><p>%1$s</p></div>', $notice );
        }
    }
    
    public function generate_freemius_licence()
    {
        if ( wp_doing_ajax() ) {
            return;
        }
        if ( !isset( $_REQUEST['action'] ) || 'freemius' !== $_REQUEST['action'] ) {
            return;
        }
        $nonce = esc_attr( $_REQUEST['_wpnonce'] );
        if ( !wp_verify_nonce( $nonce, 'qem_freemius_licence' ) ) {
            die( __( 'Security check invalid, expired or missing', 'quick-event-manager' ) );
        }
        $user = wp_get_current_user();
        $qpp_key = get_option( 'qpp_key' );
        $request = wp_remote_get( 'https://fullworks.net/wp-json/fullworks-qpp-sync/v1/add/quick-event-manager/?key=' . $qpp_key['key'] . '&email=' . $user->data->user_email . '&domain=' . get_bloginfo( 'url' ) );
        
        if ( is_wp_error( $request ) ) {
            update_option( 'qem_licence_generated', 'failed' );
            return;
        }
        
        $response_code = wp_remote_retrieve_response_code( $request );
        if ( 200 !== $response_code ) {
            update_option( 'qem_licence_generated', 'failed' );
        }
        update_option( 'qem_licence_generated', 'generated' );
    }

}